@@ -44,6 +44,16 @@ Classes relating to configuration
 {'foo': 'bar'}
 >>> config.plugin_get('testplugin', 'foo')
 'bar'
+>>> config.plugin_get('testplugin', 'foo', 'new')
+'bar'
+>>> config.plugin_get('testplugin', 'algo')
+Traceback (most recent call last):
+...
+KeyError: 'ConfigBase::get_item: unknown key algo'
+>>> config.plugin_get('testplugin', 'algo', 1)
+1
+>>> config.plugin_get('anothertestplugin', 'algo', 500)
+500
 >>> config.get_profile()
 'default'
 >>> config.set_profile('my_first_new_testing_profile')
@@ -83,14 +93,19 @@ DEFAULTS = {
             'window_state'          : 'normal',
             'borderless'            : False,
             'tab_position'          : 'top',
+            'broadcast_default'     : 'group',
             'close_button_on_tab'   : True,
             'hide_tabbar'           : False,
             'scroll_tabbar'         : False,
+            'homogeneous_tabbar'    : True,
             'hide_from_taskbar'     : False,
             'always_on_top'         : False,
             'hide_on_lose_focus'    : False,
             'sticky'                : False,
             'try_posix_regexp'      : platform.system() != 'Linux',
+            'use_custom_url_handler': False,
+            'custom_url_handler'    : '',
+            'disable_real_transparency' : False,
             'title_hide_sizetext'   : False,
             'title_transmit_fg_color' : '#ffffff',
             'title_transmit_bg_color' : '#c80003',
@@ -98,9 +113,14 @@ DEFAULTS = {
             'title_receive_bg_color' : '#0076c9',
             'title_inactive_fg_color' : '#000000',
             'title_inactive_bg_color' : '#c0bebf',
+            'inactive_color_offset': 0.8,
             'enabled_plugins'       : ['LaunchpadBugURLHandler',
                                        'LaunchpadCodeURLHandler',
                                        'APTURLHandler'],
+            'suppress_multiple_term_dialog': False,
+            'always_split_with_profile': False,
+            'title_use_system_font' : True,
+            'title_font'            : 'Sans 9',
         },
         'keybindings': {
             'zoom_in'          : '<Control>plus',
@@ -115,6 +135,8 @@ DEFAULTS = {
             'go_down'          : '<Alt>Down',
             'go_left'          : '<Alt>Left',
             'go_right'         : '<Alt>Right',
+            'rotate_cw'        : '<Super>r',
+            'rotate_ccw'       : '<Super><Shift>r',
             'split_horiz'      : '<Shift><Control>o',
             'split_vert'       : '<Shift><Control>e',
             'close_term'       : '<Shift><Control>w',
@@ -122,6 +144,12 @@ DEFAULTS = {
             'paste'            : '<Shift><Control>v',
             'toggle_scrollbar' : '<Shift><Control>s',
             'search'           : '<Shift><Control>f',
+            'page_up'          : '',
+            'page_down'        : '',
+            'page_up_half'     : '',
+            'page_down_half'   : '',
+            'line_up'          : '',
+            'line_down'        : '',
             'close_window'     : '<Shift><Control>q',
             'resize_up'        : '<Shift><Control>Up',
             'resize_down'      : '<Shift><Control>Down',
@@ -148,14 +176,23 @@ DEFAULTS = {
             'reset_clear'      : '<Shift><Control>g',
             'hide_window'      : '<Shift><Control><Alt>a',
             'group_all'        : '<Super>g',
+            'group_all_toggle' : '',
             'ungroup_all'      : '<Shift><Super>g',
             'group_tab'        : '<Super>t',
+            'group_tab_toggle' : '',
             'ungroup_tab'      : '<Shift><Super>t',
             'new_window'       : '<Shift><Control>i',
             'new_terminator'   : '<Super>i',
-            'broadcast_off'    : '',
-            'broadcast_group'  : '',
-            'broadcast_all'    : ''
+            'broadcast_off'    : '<Alt>o',
+            'broadcast_group'  : '<Alt>g',
+            'broadcast_all'    : '<Alt>a',
+            'insert_number'    : '<Super>1',
+            'insert_padded'    : '<Super>0',
+            'edit_window_title': '<Alt>t',
+            'layout_launcher'  : '<Alt>l',
+            'next_profile'     : '',
+            'previous_profile' : '', 
+            'help'             : 'F1'
         },
         'profiles': {
             'default':  {
@@ -176,6 +213,8 @@ DEFAULTS = {
                 'cursor_shape'          : 'block',
                 'cursor_color'          : '#aaaaaa',
                 'emulation'             : 'xterm',
+                'term'                  : 'xterm',
+                'colorterm'             : 'gnome-terminal',
                 'font'                  : 'Mono 10',
                 'foreground_color'      : '#aaaaaa',
                 'show_titlebar'         : True,
@@ -231,7 +270,8 @@ class Config(object):
     base = None
     profile = None
     gconf = None
-    system_font = None
+    system_mono_font = None
+    system_prop_font = None
     system_focus = None
     inhibited = None
     
@@ -240,9 +280,9 @@ class Config(object):
         self.set_profile(profile)
         self.inhibited = False
 
-    def __getitem__(self, key):
+    def __getitem__(self, key, default=None):
         """Look up a configuration item"""
-        return(self.base.get_item(key, self.profile))
+        return(self.base.get_item(key, self.profile, default=default))
 
     def __setitem__(self, key, value):
         """Set a particular configuration item"""
@@ -298,6 +338,10 @@ class Config(object):
         """Add a new layout"""
         return(self.base.add_layout(name, layout))
 
+    def replace_layout(self, name, layout):
+        """Replace an existing layout"""
+        return(self.base.replace_layout(name, layout)) 
+
     def del_layout(self, layout):
         """Delete a layout"""
         if self.base.layouts.has_key(layout):
@@ -313,10 +357,31 @@ class Config(object):
         """List all configured layouts"""
         return(self.base.layouts.keys())
 
-    def get_system_font(self):
+    def get_system_prop_font(self):
         """Look up the system font"""
-        if self.system_font is not None:
-            return(self.system_font)
+        if self.system_prop_font is not None:
+            return(self.system_prop_font)
+        elif 'gconf' not in globals():
+            return
+        else:
+            if self.gconf is None:
+                self.gconf = gconf.client_get_default()
+
+            value = self.gconf.get(
+                        '/desktop/gnome/interface/font_name')
+            if value:
+                self.system_prop_font = value.get_string()
+            else:
+                self.system_prop_font = "Sans 10"
+            self.gconf.notify_add(
+                        '/desktop/gnome/interface/font_name', 
+                        self.on_gconf_notify)
+            return(self.system_prop_font)
+
+    def get_system_mono_font(self):
+        """Look up the system font"""
+        if self.system_mono_font is not None:
+            return(self.system_mono_font)
         elif 'gconf' not in globals():
             return
         else:
@@ -325,11 +390,14 @@ class Config(object):
 
             value = self.gconf.get(
                         '/desktop/gnome/interface/monospace_font_name')
-            self.system_font = value.get_string()
+            if value:
+                self.system_mono_font = value.get_string()
+            else:
+                self.system_mono_font = "Mono 10"
             self.gconf.notify_add(
                         '/desktop/gnome/interface/monospace_font_name', 
                         self.on_gconf_notify)
-            return(self.system_font)
+            return(self.system_mono_font)
 
     def get_system_focus(self):
         """Look up the system focus setting"""
@@ -342,9 +410,10 @@ class Config(object):
                 self.gconf = gconf.client_get_default()
 
             value = self.gconf.get('/apps/metacity/general/focus_mode')
-            self.system_focus = value.get_string()
-            self.gconf.notify_add('/apps/metacity/general/focus_mode',
-                    self.on_gconf_notify)
+            if value:
+                self.system_focus = value.get_string()
+                self.gconf.notify_add('/apps/metacity/general/focus_mode',
+                        self.on_gconf_notify)
             return(self.system_focus)
 
     def on_gconf_notify(self, _client, _cnxn_id, _entry, _what):
@@ -376,9 +445,11 @@ class Config(object):
         """Get the command line options"""
         return(self.base.command_line_options)
 
-    def plugin_get(self, pluginname, key):
-        """Get a plugin config value"""
-        return(self.base.get_item(key, plugin=pluginname))
+    def plugin_get(self, pluginname, key, default=None):
+        """Get a plugin config value, if doesn't exist
+            return default if specified
+        """
+        return(self.base.get_item(key, plugin=pluginname, default=default))
 
     def plugin_set(self, pluginname, key, value):
         """Set a plugin config value"""
@@ -392,6 +463,10 @@ class Config(object):
         """Set a whole config tree for a given plugin"""
         return(self.base.set_plugin(plugin, tree))
 
+    def plugin_del_config(self, plugin):
+        """Delete a whole config tree for a given plugin"""
+        return(self.base.del_plugin(plugin))
+
     def layout_get_config(self, layout):
         """Return a layout"""
         return(self.base.get_layout(layout))
@@ -418,6 +493,8 @@ class ConfigBase(Borg):
         Borg.__init__(self, self.__class__.__name__)
 
         self.prepare_attributes()
+        import optionparse
+        self.command_line_options = optionparse.options
         self.load()
 
     def prepare_attributes(self):
@@ -465,6 +542,9 @@ class ConfigBase(Borg):
 
             keytype = '%s(default=%s)' % (keytype, value)
 
+            if key == 'custom_url_handler':
+                keytype = 'string(default="")'
+
             section[key] = keytype
         configspecdata['global_config'] = section
 
@@ -484,6 +564,8 @@ class ConfigBase(Borg):
                 keytype = keymap[keytype]
             elif keytype == 'list':
                 value = 'list(%s)' % ','.join(value)
+            if key == 'background_image':
+                keytype = 'string'
             if keytype == 'string':
                 value = '"%s"' % value
 
@@ -517,7 +599,13 @@ class ConfigBase(Borg):
             dbg('ConfigBase::load: config already loaded')
             return
 
-        filename = os.path.join(get_config_dir(), 'config')
+        if self.command_line_options:
+            if not self.command_line_options.config:
+                self.command_line_options.config = os.path.join(get_config_dir(), 'config')
+            filename = self.command_line_options.config
+        else:
+            filename = os.path.join(get_config_dir(), 'config')
+
         dbg('looking for config file: %s' % filename)
         try:
             configfile = open(filename, 'r')
@@ -581,6 +669,11 @@ class ConfigBase(Borg):
                             section_name)
 
         self.loaded = True
+
+    def reload(self):
+        """Force a reload of the base config"""
+        self.loaded = False
+        self.load()
         
     def save(self):
         """Save the config to a file"""
@@ -613,11 +706,11 @@ class ConfigBase(Borg):
         if not os.path.isdir(config_dir):
             os.makedirs(config_dir)
         try:
-            parser.write(open(os.path.join(config_dir, 'config'), 'w'))
+            parser.write(open(self.command_line_options.config, 'w'))
         except Exception, ex:
             err('ConfigBase::save: Unable to save config: %s' % ex)
 
-    def get_item(self, key, profile='default', plugin=None):
+    def get_item(self, key, profile='default', plugin=None, default=None):
         """Look up a configuration item"""
         if not self.profiles.has_key(profile):
             # Hitting this generally implies a bug
@@ -633,10 +726,12 @@ class ConfigBase(Borg):
             return(self.profiles[profile][key])
         elif key == 'keybindings':
             return(self.keybindings)
-        elif plugin is not None and self.plugins[plugin].has_key(key):
+        elif plugin and plugin in self.plugins and key in self.plugins[plugin]:
             dbg('ConfigBase::get_item: %s found in plugin %s: %s' % (
                     key, plugin, self.plugins[plugin][key]))
             return(self.plugins[plugin][key])
+        elif default:
+            return default
         else:
             raise KeyError('ConfigBase::get_item: unknown key %s' % key)
 
@@ -669,6 +764,11 @@ class ConfigBase(Borg):
         """Set a whole tree for a plugin"""
         self.plugins[plugin] = tree
 
+    def del_plugin(self, plugin):
+        """Delete a whole tree for a plugin"""
+        if plugin in self.plugins:
+            del self.plugins[plugin]
+
     def add_profile(self, profile):
         """Add a new profile"""
         if profile in self.profiles:
@@ -683,6 +783,13 @@ class ConfigBase(Borg):
         self.layouts[name] = layout
         return(True)
 
+    def replace_layout(self, name, layout):
+        """Replaces a layout with the given name"""
+        if not name in self.layouts:
+            return(False)
+        self.layouts[name] = layout
+        return(True)
+
     def get_layout(self, layout):
         """Return a layout"""
         if self.layouts.has_key(layout):
