@@ -80,6 +80,10 @@ class Container(object):
         """Default axis splitter. This should be implemented by subclasses"""
         raise NotImplementedError('split_axis')
 
+    def rotate(self, widget, clockwise):
+        """Rotate children in this container"""
+        raise NotImplementedError('rotate')
+
     def add(self, widget, metadata=None):
         """Add a widget to the container"""
         raise NotImplementedError('add')
@@ -151,6 +155,11 @@ class Container(object):
 
     def construct_confirm_close(self, window, reqtype):
         """Create a confirmation dialog for closing things"""
+        
+        # skip this dialog if applicable
+        if self.config['suppress_multiple_term_dialog']:
+            return gtk.RESPONSE_ACCEPT
+        
         dialog = gtk.Dialog(_('Close?'), window, gtk.DIALOG_MODAL)
         dialog.set_has_separator(False)
         dialog.set_resizable(False)
@@ -166,7 +175,7 @@ class Container(object):
         secondary = gtk.Label(_('This %s has several terminals open. Closing \
 the %s will also close all terminals within it.') % (reqtype, reqtype))
         secondary.set_line_wrap(True)
-    
+                    
         labels = gtk.VBox()
         labels.pack_start(primary, False, False, 6)
         labels.pack_start(secondary, False, False, 6)
@@ -179,9 +188,22 @@ the %s will also close all terminals within it.') % (reqtype, reqtype))
         box.pack_start(image, False, False, 6)
         box.pack_start(labels, False, False, 6)
         dialog.vbox.pack_start(box, False, False, 12)
+
+        checkbox = gtk.CheckButton(_("Do not show this message next time"))
+        dialog.vbox.pack_end(checkbox)
     
         dialog.show_all()
-        return(dialog)
+
+        result = dialog.run()
+        
+        # set configuration
+        self.config.base.reload()
+        self.config['suppress_multiple_term_dialog'] = checkbox.get_active()
+        self.config.save()
+
+        dialog.destroy()
+                
+        return(result)
 
     def propagate_title_change(self, widget, title):
         """Pass a title change up the widget stack"""
@@ -235,18 +257,42 @@ the %s will also close all terminals within it.') % (reqtype, reqtype))
             if hasattr(position, '__iter__'):
                 position = ':'.join([str(x) for x in position])
             layout['position'] = position
+        
+        if hasattr(self, 'ismaximised'):
+            layout['maximised'] = self.ismaximised
+        
+        if hasattr(self, 'isfullscreen'):
+            layout['fullscreen'] = self.isfullscreen
+        
+        if hasattr(self, 'ratio'):
+            layout['ratio'] = self.ratio
 
         if hasattr(self, 'get_size'):
             layout['size'] = self.get_size()
 
-        labels = []
+        if hasattr(self, 'title'):
+            layout['title'] = self.title.text
+
         if mytype == 'Notebook':
+            labels = []
+            last_active_term = []
             for tabnum in xrange(0, self.get_n_pages()):
                 page = self.get_nth_page(tabnum)
                 label = self.get_tab_label(page)
                 labels.append(label.get_custom_label())
-        if len(labels) > 0:
+                last_active_term.append(self.last_active_term[self.get_nth_page(tabnum)])
             layout['labels'] = labels
+            layout['last_active_term'] = last_active_term
+            layout['active_page'] = self.get_current_page()
+        else:
+            if hasattr(self, 'last_active_term') and self.last_active_term is not None:
+                layout['last_active_term'] = self.last_active_term
+
+        if mytype == 'Window':
+            if self.uuid == self.terminator.last_active_window:
+                layout['last_active_window'] = True
+            else:
+                layout['last_active_window'] = False
 
         name = 'child%d' % count
         count = count + 1
@@ -265,4 +311,5 @@ the %s will also close all terminals within it.') % (reqtype, reqtype))
         """Apply settings for our layout"""
         raise NotImplementedError('create_layout')
 
+
 # vim: set expandtab ts=4 sw=4:
