@@ -1,4 +1,4 @@
-#!/usr/local/bin/python
+#!/usr/bin/python
 #
 # Copyright (c) 2008, Thomas Hurst <tom@hur.st>
 #
@@ -43,7 +43,7 @@ class kinfo_file(Structure):
       ('kf_sa_peer',       sockaddr_storage),
   ]
 
-libc = CDLL('libc.so')
+libc = CDLL('libc.so.7')
 
 uintlen = c_size_t(sizeof(c_uint))
 ver = c_uint(0)
