@@ -26,6 +26,8 @@ import config
 import version
 from translation import _
 
+options = None
+
 def execute_cb(option, opt, value, lparser):
     """Callback for use in parsing execute options"""
     assert value is None
@@ -40,7 +42,8 @@ def parse_options():
     """Parse the command line options"""
     usage = "usage: %prog [options]"
 
-    configobj = config.Config()
+    is_x_terminal_emulator = os.path.basename(sys.argv[0]) == 'x-terminal-emulator'
+
     parser = OptionParser(usage)
 
     parser.add_option('-v', '--version', action='store_true', dest='version',
@@ -53,22 +56,41 @@ def parse_options():
             dest='borderless', help=_('Disable window borders'))
     parser.add_option('-H', '--hidden', action='store_true', dest='hidden',
             help=_('Hide the window at startup'))
-    parser.add_option('-T', '--title', dest='forcedtitle', help=_('Specify a \
-title for the window'))
-    parser.add_option('--geometry', dest='geometry', type='string', help=_('Set \
-the preferred size and position of the window (see X man page)'))
-    parser.add_option('-e', '--command', dest='command', help=_('Specify a \
-command to execute inside the terminal'))
+    parser.add_option('-T', '--title', dest='forcedtitle', 
+                      help=_('Specify a title for the window'))
+    parser.add_option('--geometry', dest='geometry', type='string', 
+                      help=_('Set the preferred size and position of the window'
+                             '(see X man page)'))
+    if not is_x_terminal_emulator:
+        parser.add_option('-e', '--command', dest='command', 
+                help=_('Specify a command to execute inside the terminal'))
+    else:
+        parser.add_option('--command', dest='command', 
+                help=_('Specify a command to execute inside the terminal'))
+        parser.add_option('-e', '--execute2', dest='execute', action='callback',
+                callback=execute_cb, 
+                help=_('Use the rest of the command line as a command to '
+                       'execute inside the terminal, and its arguments'))
+    parser.add_option('-g', '--config', dest='config', 
+                      help=_('Specify a config file'))
     parser.add_option('-x', '--execute', dest='execute', action='callback',
-            callback=execute_cb, help=_('Use the rest of the command line as a \
-command to execute inside the terminal, and its arguments'))
+            callback=execute_cb, 
+            help=_('Use the rest of the command line as a command to execute '
+                   'inside the terminal, and its arguments'))
     parser.add_option('--working-directory', metavar='DIR',
             dest='working_directory', help=_('Set the working directory'))
-    parser.add_option('-r', '--role', dest='role', help=_('Set a custom \
-WM_WINDOW_ROLE property on the window'))
-    parser.add_option('-l', '--layout', dest='layout', help=_('Select a layout'))
-    parser.add_option('-p', '--profile', dest='profile', help=_('Use a \
-different profile as the default'))
+    parser.add_option('-c', '--classname', dest='classname', help=_('Set a \
+custom name (WM_CLASS) property on the window'))
+    parser.add_option('-i', '--icon', dest='forcedicon', help=_('Set a custom \
+icon for the window (by file or name)'))
+    parser.add_option('-r', '--role', dest='role', 
+            help=_('Set a custom WM_WINDOW_ROLE property on the window'))
+    parser.add_option('-l', '--layout', dest='layout', 
+            help=_('Launch with the given layout'))
+    parser.add_option('-s', '--select-layout', action='store_true',
+            dest='select', help=_('Select a layout from a list'))
+    parser.add_option('-p', '--profile', dest='profile', 
+            help=_('Use a different profile as the default'))
     parser.add_option('-u', '--no-dbus', action='store_true', dest='nodbus', 
             help=_('Disable DBus'))
     parser.add_option('-d', '--debug', action='count', dest='debug',
@@ -77,11 +99,14 @@ different profile as the default'))
             help=_('Comma separated list of classes to limit debugging to'))
     parser.add_option('--debug-methods', action='store', dest='debug_methods',
             help=_('Comma separated list of methods to limit debugging to'))
+    parser.add_option('--new-tab', action='store_true', dest='new_tab',
+            help=_('If Terminator is already running, just open a new tab'))
     for item in ['--sm-client-id', '--sm-config-prefix', '--screen', '-n',
-    '--no-gconf' ]:
+                 '--no-gconf' ]:
         parser.add_option(item, dest='dummy', action='store',
                 help=SUPPRESS_HELP)
 
+    global options
     (options, args) = parser.parse_args()
     if len(args) != 0:
         parser.error('Additional unexpected arguments found: %s' % args)
@@ -118,6 +143,7 @@ different profile as the default'))
     if options.layout is None:
         options.layout = 'default'
 
+    configobj = config.Config()
     if options.profile and options.profile not in configobj.list_profiles():
         options.profile = None
 
