@@ -1,5 +1,5 @@
 #!/usr/bin/python
-"""Preferences Editor for Terminator. 
+"""Preferences Editor for Terminator.
 
 Load a UIBuilder config file, display it,
 populate it with our current config, then optionally read that back out and
@@ -9,6 +9,7 @@ write it to a config file
 
 import os
 import gtk
+import gobject
 
 from util import dbg, err
 import config
@@ -17,6 +18,7 @@ from translation import _
 from encoding import TerminatorEncoding
 from terminator import Terminator
 from plugin import PluginRegistry
+from version import APP_NAME
 
 def color2hex(widget):
     """Pull the colour values out of a Gtk ColorPicker widget and return them
@@ -36,104 +38,131 @@ class PrefsEditor:
     layouteditor = None
     previous_layout_selection = None
     previous_profile_selection = None
-    colorschemevalues = {'black_on_yellow': 0, 
+    colorschemevalues = {'black_on_yellow': 0,
                          'black_on_white': 1,
                          'grey_on_black': 2,
                          'green_on_black': 3,
                          'white_on_black': 4,
                          'orange_on_black': 5,
                          'ambience': 6,
-                         'custom': 7}
+                         'solarized_light': 7,
+                         'solarized_dark': 8,
+                         'custom': 9}
     colourschemes = {'grey_on_black': ['#aaaaaa', '#000000'],
                      'black_on_yellow': ['#000000', '#ffffdd'],
                      'black_on_white': ['#000000', '#ffffff'],
                      'white_on_black': ['#ffffff', '#000000'],
                      'green_on_black': ['#00ff00', '#000000'],
                      'orange_on_black': ['#e53c00', '#000000'],
-                     'ambience': ['#ffffff', '#300a24']}
+                     'ambience': ['#ffffff', '#300a24'],
+                     'solarized_light': ['#657b83', '#fdf6e3'],
+                     'solarized_dark': ['#839496', '#002b36']}
     palettevalues = {'tango': 0,
                      'linux': 1,
                      'xterm': 2,
                      'rxvt': 3,
                      'ambience': 4,
-                     'custom': 5}
+                     'solarized': 5,
+                     'custom': 6}
     palettes = {'tango': '#000000:#cc0000:#4e9a06:#c4a000:#3465a4:\
 #75507b:#06989a:#d3d7cf:#555753:#ef2929:#8ae234:#fce94f:#729fcf:\
 #ad7fa8:#34e2e2:#eeeeec',
                 'linux': '#000000:#aa0000:#00aa00:#aa5500:#0000aa:\
 #aa00aa:#00aaaa:#aaaaaa:#555555:#ff5555:#55ff55:#ffff55:#5555ff:\
 #ff55ff:#55ffff:#ffffff',
-                'xterm': '#000000:#cd0000:#00cd00:#cdcd00:#1e90ff:\
-#cd00cd:#00cdcd:#e5e5e5:#4c4c4c:#ff0000:#00ff00:#ffff00:#4682b4:\
+                'xterm': '#000000:#cd0000:#00cd00:#cdcd00:#0000ee:\
+#cd00cd:#00cdcd:#e5e5e5:#7f7f7f:#ff0000:#00ff00:#ffff00:#5c5cff:\
 #ff00ff:#00ffff:#ffffff',
                 'rxvt': '#000000:#cd0000:#00cd00:#cdcd00:#0000cd:\
 #cd00cd:#00cdcd:#faebd7:#404040:#ff0000:#00ff00:#ffff00:#0000ff:\
 #ff00ff:#00ffff:#ffffff',
                 'ambience': '#2e3436:#cc0000:#4e9a06:#c4a000:\
 #3465a4:#75507b:#06989a:#d3d7cf:#555753:#ef2929:#8ae234:#fce94f:\
-#729fcf:#ad7fa8:#34e2e2:#eeeeec'}
-    keybindingnames = { 'zoom_in'          : 'Increase font size',
-                        'zoom_out'         : 'Decrease font size',
-                        'zoom_normal'      : 'Restore original font size',
-                        'new_tab'          : 'Create a new tab',
-                        'cycle_next'       : 'Focus the next terminal',
-                        'cycle_prev'       : 'Focus the previous terminal',
-                        'go_next'          : 'Focus the next terminal',
-                        'go_prev'          : 'Focus the previous terminal',
-                        'go_up'            : 'Focus the terminal above',
-                        'go_down'          : 'Focus the terminal below',
-                        'go_left'          : 'Focus the terminal left',
-                        'go_right'         : 'Focus the terminal right',
-                        'split_horiz'      : 'Split horizontally',
-                        'split_vert'       : 'Split vertically',
-                        'close_term'       : 'Close terminal',
-                        'copy'             : 'Copy selected text',
-                        'paste'            : 'Paste clipboard',
-                        'toggle_scrollbar' : 'Show/Hide the scrollbar',
-                        'search'           : 'Search terminal scrollback',
-                        'close_window'     : 'Close window',
-                        'resize_up'        : 'Resize the terminal up',
-                        'resize_down'      : 'Resize the terminal down',
-                        'resize_left'      : 'Resize the terminal left',
-                        'resize_right'     : 'Resize the terminal right',
-                        'move_tab_right'   : 'Move the tab right',
-                        'move_tab_left'    : 'Move the tab left',
-                        'toggle_zoom'      : 'Maximise terminal',
-                        'scaled_zoom'      : 'Zoom terminal',
-                        'next_tab'         : 'Switch to the next tab',
-                        'prev_tab'         : 'Switch to the previous tab',
-                        'switch_to_tab_1'  : 'Switch to the first tab',
-                        'switch_to_tab_2'  : 'Switch to the second tab',
-                        'switch_to_tab_3'  : 'Switch to the third tab',
-                        'switch_to_tab_4'  : 'Switch to the fourth tab',
-                        'switch_to_tab_5'  : 'Switch to the fifth tab',
-                        'switch_to_tab_6'  : 'Switch to the sixth tab',
-                        'switch_to_tab_7'  : 'Switch to the seventh tab',
-                        'switch_to_tab_8'  : 'Switch to the eighth tab',
-                        'switch_to_tab_9'  : 'Switch to the ninth tab',
-                        'switch_to_tab_10' : 'Switch to the tenth tab',
-                        'full_screen'      : 'Toggle fullscreen',
-                        'reset'            : 'Reset the terminal',
-                        'reset_clear'      : 'Reset and clear the terminal',
-                        'hide_window'      : 'Toggle window visibility',
-                        'group_all'        : 'Group all terminals',
-                        'ungroup_all'      : 'Ungroup all terminals',
-                        'group_tab'        : 'Group terminals in tab',
-                        'ungroup_tab'      : 'Ungroup terminals in tab',
-                        'new_window'       : 'Create a new window',
-                        'new_terminator'   : 'Spawn a new Terminator process',
-                        'broadcast_off'    : 'Don\'t broadcast key presses',
-                        'broadcast_group'  : 'Broadcast key presses to group',
-                        'broadcast_all'    : 'Broadcast key events to all'
+#729fcf:#ad7fa8:#34e2e2:#eeeeec',
+                'solarized': '#073642:#dc322f:#859900:#b58900:\
+#268bd2:#d33682:#2aa198:#eee8d5:#002b36:#cb4b16:#586e75:#657b83:\
+#839496:#6c71c4:#93a1a1:#fdf6e3'}
+    keybindingnames = { 'zoom_in'          : _('Increase font size'),
+                        'zoom_out'         : _('Decrease font size'),
+                        'zoom_normal'      : _('Restore original font size'),
+                        'new_tab'          : _('Create a new tab'),
+                        'cycle_next'       : _('Focus the next terminal'),
+                        'cycle_prev'       : _('Focus the previous terminal'),
+                        'go_next'          : _('Focus the next terminal'),
+                        'go_prev'          : _('Focus the previous terminal'),
+                        'go_up'            : _('Focus the terminal above'),
+                        'go_down'          : _('Focus the terminal below'),
+                        'go_left'          : _('Focus the terminal left'),
+                        'go_right'         : _('Focus the terminal right'),
+                        'rotate_cw'        : _('Rotate terminals clockwise'),
+                        'rotate_ccw'       : _('Rotate terminals counter-clockwise'),
+                        'split_horiz'      : _('Split horizontally'),
+                        'split_vert'       : _('Split vertically'),
+                        'close_term'       : _('Close terminal'),
+                        'copy'             : _('Copy selected text'),
+                        'paste'            : _('Paste clipboard'),
+                        'toggle_scrollbar' : _('Show/Hide the scrollbar'),
+                        'search'           : _('Search terminal scrollback'),
+                        'page_up'          : _('Scroll upwards one page'),
+                        'page_down'        : _('Scroll downwards one page'),
+                        'page_up_half'     : _('Scroll upwards half a page'),
+                        'page_down_half'   : _('Scroll downwards half a page'),
+                        'line_up'          : _('Scroll upwards one line'),
+                        'line_down'        : _('Scroll downwards one line'),
+                        'close_window'     : _('Close window'),
+                        'resize_up'        : _('Resize the terminal up'),
+                        'resize_down'      : _('Resize the terminal down'),
+                        'resize_left'      : _('Resize the terminal left'),
+                        'resize_right'     : _('Resize the terminal right'),
+                        'move_tab_right'   : _('Move the tab right'),
+                        'move_tab_left'    : _('Move the tab left'),
+                        'toggle_zoom'      : _('Maximise terminal'),
+                        'scaled_zoom'      : _('Zoom terminal'),
+                        'next_tab'         : _('Switch to the next tab'),
+                        'prev_tab'         : _('Switch to the previous tab'),
+                        'switch_to_tab_1'  : _('Switch to the first tab'),
+                        'switch_to_tab_2'  : _('Switch to the second tab'),
+                        'switch_to_tab_3'  : _('Switch to the third tab'),
+                        'switch_to_tab_4'  : _('Switch to the fourth tab'),
+                        'switch_to_tab_5'  : _('Switch to the fifth tab'),
+                        'switch_to_tab_6'  : _('Switch to the sixth tab'),
+                        'switch_to_tab_7'  : _('Switch to the seventh tab'),
+                        'switch_to_tab_8'  : _('Switch to the eighth tab'),
+                        'switch_to_tab_9'  : _('Switch to the ninth tab'),
+                        'switch_to_tab_10' : _('Switch to the tenth tab'),
+                        'full_screen'      : _('Toggle fullscreen'),
+                        'reset'            : _('Reset the terminal'),
+                        'reset_clear'      : _('Reset and clear the terminal'),
+                        'hide_window'      : _('Toggle window visibility'),
+                        'group_all'        : _('Group all terminals'),
+                        'group_all_toggle' : _('Group/Ungroup all terminals'),
+                        'ungroup_all'      : _('Ungroup all terminals'),
+                        'group_tab'        : _('Group terminals in tab'),
+                        'group_tab_toggle' : _('Group/Ungroup terminals in tab'),
+                        'ungroup_tab'      : _('Ungroup terminals in tab'),
+                        'new_window'       : _('Create a new window'),
+                        'new_terminator'   : _('Spawn a new Terminator process'),
+                        'broadcast_off'    : _('Don\'t broadcast key presses'),
+                        'broadcast_group'  : _('Broadcast key presses to group'),
+                        'broadcast_all'    : _('Broadcast key events to all'),
+                        'insert_number'    : _('Insert terminal number'),
+                        'insert_padded'    : _('Insert padded terminal number'),
+                        'edit_window_title': _('Edit window title'),
+                        'layout_launcher'  : _('Open layout launcher window'),
+                        'next_profile'     : _('Switch to next profile'),
+                        'previous_profile' : _('Switch to previous profile'), 
+                        'help'             : _('Open the manual')
             }
 
     def __init__ (self, term):
         self.config = config.Config()
+        self.config.base.reload()
         self.term = term
         self.builder = gtk.Builder()
+        self.builder.set_translation_domain(APP_NAME)
         self.keybindings = Keybindings()
         try:
-            # Figure out where our library is on-disk so we can open our 
+            # Figure out where our library is on-disk so we can open our
             (head, _tail) = os.path.split(config.__file__)
             librarypath = os.path.join(head, 'preferences.glade')
             gladefile = open(librarypath, 'r')
@@ -145,6 +174,15 @@ class PrefsEditor:
 
         self.builder.add_from_string(gladedata)
         self.window = self.builder.get_object('prefswin')
+
+        icon_theme = gtk.icon_theme_get_default()
+        if icon_theme.lookup_icon('terminator-preferences', 48, 0):
+            self.window.set_icon_name('terminator-preferences')
+        else:
+            dbg('Unable to load Terminator preferences icon')
+            icon = self.window.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
+            self.window.set_icon(icon)
+
         self.layouteditor = LayoutEditor(self.builder)
         self.builder.connect_signals(self)
         self.layouteditor.prepare()
@@ -215,9 +253,56 @@ class PrefsEditor:
         else:
             active = 0
         widget.set_active(active)
+        # Broadcast default
+        option = self.config['broadcast_default']
+        widget = guiget('broadcastdefault')
+        if option == 'all':
+            active = 0
+        elif option == 'off':
+            active = 2
+        else:
+            active = 1
+        widget.set_active(active)
+        # scroll_tabbar
+        widget = guiget('scrolltabbarcheck')
+        widget.set_active(self.config['scroll_tabbar'])
+        # homogeneous_tabbar
+        widget = guiget('homogeneouscheck')
+        widget.set_active(self.config['homogeneous_tabbar'])
         # DBus Server
         widget = guiget('dbuscheck')
         widget.set_active(self.config['dbus'])
+        #Hide from taskbar
+        widget = guiget('hidefromtaskbcheck')
+        widget.set_active(self.config['hide_from_taskbar'])
+        #Always on top
+        widget = guiget('alwaysontopcheck')
+        widget.set_active(self.config['always_on_top'])
+        #Hide on lose focus
+        widget = guiget('hideonlosefocuscheck')
+        widget.set_active(self.config['hide_on_lose_focus'])
+        #Show on all workspaces
+        widget = guiget('stickycheck')
+        widget.set_active(self.config['sticky'])
+        #Hide size text from the title bar
+        widget = guiget('title_hide_sizetextcheck')
+        widget.set_active(self.config['title_hide_sizetext'])
+        #Always split with profile
+        widget = guiget('always_split_with_profile')
+        widget.set_active(self.config['always_split_with_profile'])
+        #Titlebar font selector
+        # Use system font
+        widget = guiget('title_system_font_checkbutton')
+        widget.set_active(self.config['title_use_system_font'])
+        self.on_title_system_font_checkbutton_toggled(widget)
+        # Font selector
+        widget = guiget('title_font_selector')
+        if self.config['title_use_system_font'] == True:
+            fontname = self.config.get_system_prop_font()
+            if fontname is not None:
+                widget.set_font_name(fontname)
+        else:
+            widget.set_font_name(self.config['title_font'])
 
         ## Profile tab
         # Populate the profile list
@@ -248,7 +333,12 @@ class PrefsEditor:
             self.layoutiters[layout] = liststore.append([layout, editable])
         selection = widget.get_selection()
         selection.connect('changed', self.on_layout_selection_changed)
-        selection.select_iter(self.layoutiters['default'])
+        terminator = Terminator()
+        if terminator.layoutname:
+            layout_to_highlight = terminator.layoutname
+        else:
+            layout_to_highlight = 'default'
+        selection.select_iter(self.layoutiters[layout_to_highlight])
         # Now set up the selection changed handler for the layout itself
         widget = guiget('LayoutTreeView')
         selection = widget.get_selection()
@@ -268,7 +358,7 @@ class PrefsEditor:
                     (keyval, mask) = self.keybindings._parsebinding(value)
                 except KeymapError:
                     pass
-            liststore.append([keybinding, self.keybindingnames[keybinding], 
+            liststore.append([keybinding, self.keybindingnames[keybinding],
                              keyval, mask])
 
         ## Plugins tab
@@ -306,7 +396,7 @@ class PrefsEditor:
         widget = guiget('font_selector')
 
         if self.config['use_system_font'] == True:
-            fontname = self.config.get_system_font()
+            fontname = self.config.get_system_mono_font()
             if fontname is not None:
                 widget.set_font_name(fontname)
         else:
@@ -352,7 +442,11 @@ class PrefsEditor:
         widget.set_active(self.config['cursor_blink'])
         # Cursor colour
         widget = guiget('cursor_color')
-        widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+        try:
+            widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
+        except ValueError:
+            self.config['cursor_color'] = "#FFFFFF"
+            widget.set_color(gtk.gdk.Color(self.config['cursor_color']))
 
         ## Command tab
         # Login shell
@@ -442,6 +536,16 @@ class PrefsEditor:
             'title_inactive_fg_color', 'title_inactive_bg_color']:
             widget = guiget(bit)
             widget.set_color(gtk.gdk.Color(self.config[bit]))
+        # Inactive terminal shading
+        widget = guiget('inactive_color_offset')
+        widget.set_value(float(self.config['inactive_color_offset']))
+        # Use custom URL handler
+        widget = guiget('use_custom_url_handler_checkbox')
+        widget.set_active(self.config['use_custom_url_handler'])
+        self.on_use_custom_url_handler_checkbutton_toggled(widget)
+        # Custom URL handler
+        widget = guiget('custom_url_handler_entry')
+        widget.set_text(self.config['custom_url_handler'])
 
         ## Background tab
         # Radio values
@@ -525,7 +629,7 @@ class PrefsEditor:
         for encoding in encodings:
             if encoding[1] is None:
                 continue
-            
+
             label = "%s %s" % (encoding[2], encoding[1])
             rowiter = encodingstore.append([label, encoding[1]])
 
@@ -541,6 +645,23 @@ class PrefsEditor:
         self.config['geometry_hinting'] = widget.get_active()
         self.config.save()
 
+    def on_homogeneous_toggled(self, widget):
+        """homogeneous_tabbar setting changed"""
+        guiget = self.builder.get_object
+        self.config['homogeneous_tabbar'] = widget.get_active()
+        scroll_toggled = guiget('scrolltabbarcheck')
+        if widget.get_active():
+            scroll_toggled.set_sensitive(True)
+        else:
+            scroll_toggled.set_active(True)
+            scroll_toggled.set_sensitive(False)
+        self.config.save()
+
+    def on_scroll_toggled(self, widget):
+        """scroll_tabbar setting changed"""
+        self.config['scroll_tabbar'] = widget.get_active()
+        self.config.save()
+
     def on_dbuscheck_toggled(self, widget):
         """DBus server setting changed"""
         self.config['dbus'] = widget.get_active()
@@ -551,6 +672,36 @@ class PrefsEditor:
         self.config['borderless'] = not widget.get_active()
         self.config.save()
 
+    def on_hidefromtaskbcheck_toggled(self, widget):
+        """Hide from taskbar setting changed"""
+        self.config['hide_from_taskbar'] = widget.get_active()
+        self.config.save()
+
+    def on_alwaysontopcheck_toggled(self, widget):
+        """Always on top setting changed"""
+        self.config['always_on_top'] = widget.get_active()
+        self.config.save()
+
+    def on_hideonlosefocuscheck_toggled(self, widget):
+        """Hide on lose focus setting changed"""
+        self.config['hide_on_lose_focus'] = widget.get_active()
+        self.config.save()
+
+    def on_stickycheck_toggled(self, widget):
+        """Sticky setting changed"""
+        self.config['sticky'] = widget.get_active()
+        self.config.save()
+
+    def on_title_hide_sizetextcheck_toggled(self, widget):
+        """Window geometry setting changed"""
+        self.config['title_hide_sizetext'] = widget.get_active()
+        self.config.save()
+
+    def on_always_split_with_profile_toggled(self, widget):
+        """Always split with profile setting changed"""
+        self.config['always_split_with_profile'] = widget.get_active()
+        self.config.save()
+
     def on_allow_bold_checkbutton_toggled(self, widget):
         """Allow bold setting changed"""
         self.config['allow_bold'] = widget.get_active()
@@ -779,6 +930,11 @@ class PrefsEditor:
         self.config['exit_action'] = value
         self.config.save()
 
+    def on_custom_url_handler_entry_changed(self, widget):
+        """Custom URL handler value changed"""
+        self.config['custom_url_handler'] = widget.get_text()
+        self.config.save()
+
     def on_custom_command_entry_changed(self, widget):
         """Custom command value changed"""
         self.config['custom_command'] = widget.get_text()
@@ -811,6 +967,11 @@ class PrefsEditor:
         self.config['font'] = widget.get_font_name()
         self.config.save()
 
+    def on_title_font_selector_font_set(self, widget):
+        """Titlebar Font changed"""
+        self.config['title_font'] = widget.get_font_name()
+        self.config.save()
+
     def on_title_receive_bg_color_color_set(self, widget):
         """Title receive background colour changed"""
         self.config['title_receive_bg_color'] = color2hex(widget)
@@ -841,6 +1002,13 @@ class PrefsEditor:
         self.config['title_transmit_fg_color'] = color2hex(widget)
         self.config.save()
 
+    def on_inactive_color_offset_change_value(self, widget, scroll, value):
+        """Inactive color offset setting changed"""
+        if value > 1.0:
+          value = 1.0
+        self.config['inactive_color_offset'] = round(value, 2)
+        self.config.save()
+
     def on_handlesize_change_value(self, widget, scroll, value):
         """Handle size changed"""
         value = int(value)
@@ -851,7 +1019,7 @@ class PrefsEditor:
 
     def on_focuscombo_changed(self, widget):
         """Focus type changed"""
-        selected  = widget.get_active()
+        selected = widget.get_active()
         if selected == 1:
             value = 'click'
         elif selected == 2:
@@ -877,6 +1045,18 @@ class PrefsEditor:
         self.config['tab_position'] = value
         self.config.save()
 
+    def on_broadcastdefault_changed(self, widget):
+        """Broadcast default changed"""
+        selected = widget.get_active()
+        if selected == 0:
+            value = 'all'
+        elif selected == 2:
+            value = 'off'
+        else:
+            value = 'group'
+        self.config['broadcast_default'] = value
+        self.config.save()
+
     def on_winstatecombo_changed(self, widget):
         """Window state changed"""
         selected = widget.get_active()
@@ -910,7 +1090,7 @@ class PrefsEditor:
             res = model.append([newprofile, True])
             if res:
                 path = model.get_path(res)
-                treeview.set_cursor(path, focus_column=treeview.get_column(0), 
+                treeview.set_cursor(path, focus_column=treeview.get_column(0),
                                     start_editing=True)
 
         self.layouteditor.update_profiles()
@@ -946,7 +1126,7 @@ class PrefsEditor:
 
         name = _('New Layout')
         if name in values:
-            i = 1
+            i = 0
             while name in values:
                 i = i + 1
                 name = '%s %d' % (_('New Layout'), i)
@@ -960,6 +1140,21 @@ class PrefsEditor:
 
         self.config.save()
 
+    def on_layoutrefreshbutton_clicked(self, _button):
+        """Refresh the terminals status and update"""
+        terminator = Terminator()
+        current_layout = terminator.describe_layout()
+
+        guiget = self.builder.get_object
+        treeview = guiget('layoutlist')
+        selected = treeview.get_selection()
+        (model, rowiter) = selected.get_selected()
+        name = model.get_value(rowiter, 0)
+
+        if self.config.replace_layout(name, current_layout):
+            treeview.set_cursor(model.get_path(rowiter), focus_column=treeview.get_column(0), start_editing=False)
+        self.config.save()
+
     def on_layoutremovebutton_clicked(self, _button):
         """Remove a layout from the list"""
         guiget = self.builder.get_object
@@ -979,6 +1174,17 @@ class PrefsEditor:
         selection.select_iter(model.get_iter_first())
         self.config.save()
 
+    def on_use_custom_url_handler_checkbutton_toggled(self, checkbox):
+        """Toggling the use_custom_url_handler checkbox needs to alter the
+        sensitivity of the custom_url_handler entrybox"""
+        guiget = self.builder.get_object
+        widget = guiget('custom_url_handler_entry')
+        value = checkbox.get_active()
+
+        widget.set_sensitive(value)
+        self.config['use_custom_url_handler'] = value
+        self.config.save()
+
     def on_use_custom_command_checkbutton_toggled(self, checkbox):
         """Toggling the use_custom_command checkbox needs to alter the
         sensitivity of the custom_command entrybox"""
@@ -1000,6 +1206,31 @@ class PrefsEditor:
         widget.set_sensitive(not value)
         self.config['use_system_font'] = value
         self.config.save()
+        
+        if self.config['use_system_font'] == True:
+            fontname = self.config.get_system_mono_font()
+            if fontname is not None:
+                widget.set_font_name(fontname)
+        else:
+            widget.set_font_name(self.config['font'])
+
+    def on_title_system_font_checkbutton_toggled(self, checkbox):
+        """Toggling the title_use_system_font checkbox needs to alter the
+        sensitivity of the font selector"""
+        guiget = self.builder.get_object
+        widget = guiget('title_font_selector')
+        value = checkbox.get_active()
+
+        widget.set_sensitive(not value)
+        self.config['title_use_system_font'] = value
+        self.config.save()
+
+        if self.config['title_use_system_font'] == True:
+            fontname = self.config.get_system_prop_font()
+            if fontname is not None:
+                widget.set_font_name(fontname)
+        else:
+            widget.set_font_name(self.config['title_font'])
 
     def on_reset_compatibility_clicked(self, widget):
         """Reset the confusing and annoying backspace/delete options to the
@@ -1038,7 +1269,7 @@ class PrefsEditor:
         else:
             guiget('background_image_filechooser').set_sensitive(False)
             guiget('scroll_background_checkbutton').set_sensitive(False)
-        if backtype == 'transparent':
+        if backtype in ('transparent', 'image'):
             guiget('darken_background_scale').set_sensitive(True)
         else:
             guiget('darken_background_scale').set_sensitive(False)
@@ -1106,14 +1337,25 @@ class PrefsEditor:
 
     def on_profile_name_edited(self, cell, path, newtext):
         """Update a profile name"""
-        oldname = cell.get_property('text')
+        oldname_broken = cell.get_property('text')
+        
+        guiget = self.builder.get_object
+        treeview = guiget('profilelist')
+        treeselection = treeview.get_selection()
+        treeselection.select_path(path)
+        (model, pathlist) = treeselection.get_selected_rows()
+        tree_iter = model.get_iter(pathlist[0])
+        oldname = model.get_value(tree_iter,0)
+        if oldname != oldname_broken:
+            dbg('edited signal provides the wrong cell: %s != %s' %(oldname, oldname_broken))
+
         if oldname == newtext or oldname == 'default':
             return
         dbg('PrefsEditor::on_profile_name_edited: Changing %s to %s' %
         (oldname, newtext))
         self.config.rename_profile(oldname, newtext)
         self.config.save()
-        
+
         widget = self.builder.get_object('profilelist')
         model = widget.get_model()
         itera = model.get_iter(path)
@@ -1138,15 +1380,30 @@ class PrefsEditor:
         """A different command has been entered for this item"""
         self.layouteditor.on_layout_profile_command_activate(widget)
 
+    def on_layout_profile_workingdir_changed(self, widget):
+        """A different working directory has been entered for this item"""
+        self.layouteditor.on_layout_profile_workingdir_activate(widget)
+
     def on_layout_name_edited(self, cell, path, newtext):
         """Update a layout name"""
-        oldname = cell.get_property('text')
+        oldname_broken = cell.get_property('text')
+
+        guiget = self.builder.get_object
+        treeview = guiget('layoutlist')
+        treeselection = treeview.get_selection()
+        treeselection.select_path(path)
+        (model, pathlist) = treeselection.get_selected_rows()
+        tree_iter = model.get_iter(pathlist[0])
+        oldname = model.get_value(tree_iter,0)
+        if oldname != oldname_broken:
+            dbg('edited signal provides the wrong cell: %s != %s' %(oldname, oldname_broken))
+
         if oldname == newtext or oldname == 'default':
             return
         dbg('Changing %s to %s' % (oldname, newtext))
         self.config.rename_layout(oldname, newtext)
         self.config.save()
-        
+
         widget = self.builder.get_object('layoutlist')
         model = widget.get_model()
         itera = model.get_iter(path)
@@ -1234,6 +1491,10 @@ class PrefsEditor:
         self.config['keybindings'][binding] = None
         self.config.save()
 
+    def on_open_manual(self,  widget):
+        """Open the fine manual"""
+        self.term.key_help()
+
 class LayoutEditor:
     profile_ids_to_profile = None
     profile_profile_to_ids = None
@@ -1331,6 +1592,13 @@ class LayoutEditor:
         else:
             widget.set_sensitive(True)
 
+        command = self.builder.get_object('layout_profile_command')
+        chooser = self.builder.get_object('layout_profile_chooser')
+        workdir = self.builder.get_object('layout_profile_workingdir')
+        command.set_sensitive(False)
+        chooser.set_sensitive(False)
+        workdir.set_sensitive(False)
+
     def on_layout_item_selection_changed(self, selection):
         """A different item in the layout was selected"""
         (treemodel, rowiter) = selection.get_selected()
@@ -1346,14 +1614,17 @@ class LayoutEditor:
         layout_item = layout[self.layout_item]
         command = self.builder.get_object('layout_profile_command')
         chooser = self.builder.get_object('layout_profile_chooser')
+        workdir = self.builder.get_object('layout_profile_workingdir')
 
         if layout_item['type'] != 'Terminal':
             command.set_sensitive(False)
             chooser.set_sensitive(False)
+            workdir.set_sensitive(False)
             return
 
         command.set_sensitive(True)
         chooser.set_sensitive(True)
+        workdir.set_sensitive(True)
         if layout_item.has_key('command') and layout_item['command'] != '':
             command.set_text(layout_item['command'])
         else:
@@ -1364,6 +1635,11 @@ class LayoutEditor:
         else:
             chooser.set_active(0)
 
+        if layout_item.has_key('directory') and layout_item['directory'] != '':
+            workdir.set_text(layout_item['directory'])
+        else:
+            workdir.set_text('')
+
     def on_layout_profile_chooser_changed(self, widget):
         """A new profile has been selected for this item"""
         if not self.layout_item:
@@ -1380,6 +1656,13 @@ class LayoutEditor:
         layout[self.layout_item]['command'] = command
         self.config.save()
 
+    def on_layout_profile_workingdir_activate(self, widget):
+        """A new working directory has been entered for this item"""
+        workdir = widget.get_text()
+        layout = self.config.layout_get_config(self.layout_name)
+        layout[self.layout_item]['directory'] = workdir
+        self.config.save()
+
 if __name__ == '__main__':
     import util
     util.DEBUG = True
