@@ -19,6 +19,7 @@ class Terminator(Borg):
     """master object for the application"""
 
     windows = None
+    launcher_windows = None
     windowtitle = None
     terminals = None
     groups = None
@@ -33,6 +34,8 @@ class Terminator(Borg):
     debug_address = None
 
     doing_layout = None
+    layoutname = None
+    last_active_window = None
 
     groupsend = None
     groupsend_type = {'all':0, 'group':1, 'off':2}
@@ -48,14 +51,16 @@ class Terminator(Borg):
 
         if not self.windows:
             self.windows = []
+        if not self.launcher_windows:
+            self.launcher_windows = []
         if not self.terminals:
             self.terminals = []
         if not self.groups:
             self.groups = []
-        if self.groupsend == None:
-            self.groupsend = self.groupsend_type['group']
         if not self.config:
             self.config = Config()
+        if self.groupsend == None:
+            self.groupsend = self.groupsend_type[self.config['broadcast_default']]
         if not self.keybindings:
             self.keybindings = Keybindings()
             self.keybindings.configure(self.config['keybindings'])
@@ -90,7 +95,7 @@ class Terminator(Borg):
             self.gnome_client.connect('save-yourself', self.save_yourself)
             self.gnome_client.connect('die', self.die)
             dbg('GNOME session support enabled and registered')
-        except ImportError:
+        except (ImportError, AttributeError):
             self.gnome_client = False
             dbg('GNOME session support not available')
 
@@ -104,6 +109,10 @@ class Terminator(Borg):
         dbg('session manager asked us to die')
         # FIXME: Implement this
 
+    def get_windows(self):
+        """Return a list of windows"""
+        return self.windows
+
     def register_window(self, window):
         """Register a new window widget"""
         if window not in self.windows:
@@ -125,13 +134,33 @@ class Terminator(Borg):
             dbg('no windows remain, quitting')
             gtk.main_quit()
 
+    def register_launcher_window(self, window):
+        """Register a new launcher window widget"""
+        if window not in self.launcher_windows:
+            dbg('Terminator::register_launcher_window: registering %s:%s' % (id(window),
+                type(window)))
+            self.launcher_windows.append(window)
+
+    def deregister_launcher_window(self, window):
+        """de-register a launcher window widget"""
+        dbg('Terminator::deregister_launcher_window: de-registering %s:%s' %
+                (id(window), type(window)))
+        if window in self.launcher_windows:
+            self.launcher_windows.remove(window)
+        else:
+            err('%s is not in registered window list' % window)
+
+        if len(self.launcher_windows) == 0 and len(self.windows) == 0:
+            # We have no windows left, we should exit
+            dbg('no windows remain, quitting')
+            gtk.main_quit()
+
     def register_terminal(self, terminal):
         """Register a new terminal widget"""
         if terminal not in self.terminals:
             dbg('Terminator::register_terminal: registering %s:%s' %
                     (id(terminal), type(terminal)))
             self.terminals.append(terminal)
-            terminal.connect('ungroup-all', self.ungroup_all)
 
     def deregister_terminal(self, terminal):
         """De-register a terminal widget"""
@@ -234,7 +263,6 @@ class Terminator(Borg):
                 raise(ValueError)
             dbg('Creating a window')
             window, terminal = self.new_window()
-            window.create_layout(layout[windef])
             if layout[windef].has_key('position'):
                 parts = layout[windef]['position'].split(':')
                 if len(parts) == 2:
@@ -245,15 +273,80 @@ class Terminator(Borg):
                 winy = int(parts[1])
                 if winx > 1 and winy > 1:
                     window.resize(winx, winy)
+            if layout[windef].has_key('title'):
+                window.title.force_title(layout[windef]['title'])
+            if layout[windef].has_key('maximised'):
+                if layout[windef]['maximised'] == 'True':
+                    window.ismaximised = True
+                else:
+                    window.ismaximised = False
+                window.set_maximised(window.ismaximised)
+            if layout[windef].has_key('fullscreen'):
+                if layout[windef]['fullscreen'] == 'True':
+                    window.isfullscreen = True
+                else:
+                    window.isfullscreen = False
+                window.set_fullscreen(window.isfullscreen)
+            window.create_layout(layout[windef])
+
+        self.layoutname = layoutname
 
     def layout_done(self):
         """Layout operations have finished, record that fact"""
         self.doing_layout = False
+        maker = Factory()
+
+        window_last_active_term_mapping = {}
+        for window in self.windows:
+            if window.is_child_notebook():
+                source = window.get_toplevel().get_children()[0]
+            else:
+                source = window
+            window_last_active_term_mapping[window] = copy.copy(source.last_active_term)
 
         for terminal in self.terminals:
             if not terminal.pid:
                 terminal.spawn_child()
 
+        for window in self.windows:
+            if window.is_child_notebook():
+                # For windows with a notebook
+                notebook = window.get_toplevel().get_children()[0]
+                # Cycle through pages by number
+                for page in xrange(0, notebook.get_n_pages()):
+                    # Try and get the entry in the previously saved mapping
+                    mapping = window_last_active_term_mapping[window]
+                    page_last_active_term = mapping.get(notebook.get_nth_page(page),  None)
+                    if page_last_active_term is None:
+                        # Couldn't find entry, so we find the first child of type Terminal
+                        children = notebook.get_nth_page(page).get_children()
+                        for page_last_active_term in children:
+                            if maker.isinstance(page_last_active_term, 'Terminal'):
+                                page_last_active_term = page_last_active_term.uuid
+                                break
+                        else:
+                            err('Should never reach here!')
+                            page_last_active_term = None
+                    if page_last_active_term is None:
+                        # Bail on this tab as we're having no luck here, continue with the next
+                        continue
+                    # Set the notebook entry, then ensure Terminal is visible and focussed
+                    urn = page_last_active_term.urn
+                    notebook.last_active_term[notebook.get_nth_page(page)] = page_last_active_term
+                    if urn:
+                        term = self.find_terminal_by_uuid(urn)
+                        if term:
+                            term.ensure_visible_and_focussed()
+            else:
+                # For windows without a notebook ensure Terminal is visible and focussed
+                if window_last_active_term_mapping[window]:
+                    term = self.find_terminal_by_uuid(window_last_active_term_mapping[window].urn)
+                    term.ensure_visible_and_focussed()
+
+        for window in self.windows:
+            if window.uuid == self.last_active_window:
+                window.show()
+
     def reconfigure(self):
         """Update configuration for the whole application"""
 
@@ -286,15 +379,9 @@ class Terminator(Borg):
             dbg('Terminator::create_group: registering group %s' % name)
             self.groups.append(name)
 
-    def ungroup_all(self, widget):
-        """Remove all groups"""
-        for terminal in self.terminals:
-            terminal.set_group(None, None)
-        self.groups = []
-
     def closegroupedterms(self, group):
         """Close all terminals in a group"""
-        for terminal in self.terminals:
+        for terminal in self.terminals[:]:
             if terminal.group == group:
                 terminal.close()
 
@@ -349,19 +436,21 @@ class Terminator(Borg):
             idx = terminals.index(term)
             term.feed(numstr % (idx + 1))
 
+    def get_sibling_terms(self, widget):
+        termset = []
+        for term in self.terminals:
+            if term.group == widget.group:
+                termset.append(term)
+        return(termset)
+
     def get_target_terms(self, widget):
         """Get the terminals we should currently be broadcasting to"""
         if self.groupsend == self.groupsend_type['all']:
             return(self.terminals)
         elif self.groupsend == self.groupsend_type['group']:
-            termset = []
-            for term in self.terminals:
-                if term == widget or (term.group != None and term.group ==
-                        widget.group):
-                    termset.append(term)
-            return(termset)
-        else:
-            return([widget])
+            if widget.group != None:
+                return(self.get_sibling_terms(widget))
+        return([widget])
 
     def get_focussed_terminal(self):
         """iterate over all the terminals to find which, if any, has focus"""
@@ -376,6 +465,9 @@ class Terminator(Borg):
             terminal.titlebar.update(widget)
         return
 
+    def focus_left(self, widget):
+        self.last_focused_term=widget
+
     def describe_layout(self):
         """Describe our current layout"""
         layout = {}
