@@ -14,20 +14,15 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-"""Terminator.util - misc utility functions
-
->>> a = {'foo': 'bar', 'baz': 'bjonk'}
->>> b = {'foo': 'far', 'baz': 'bjonk'}
->>> dict_diff(a, b)
-{'foo': 'far'}
-
-"""
+"""Terminator.util - misc utility functions"""
 
 import sys
 import gtk
 import os
 import pwd
 import inspect
+import uuid
+import subprocess
 
 # set this to true to enable debugging output
 DEBUG = False
@@ -89,6 +84,26 @@ def has_ancestor(widget, wtype):
             return(True)
     return(False)
 
+def manual_lookup():
+    '''Choose the manual to open based on LANGUAGE'''
+    prefix = os.path.join(os.sep, 'usr', 'share', 'doc', 'terminator')
+    if 'LANGUAGE' in os.environ:
+        languages = os.environ['LANGUAGE'].split(':')
+        for language in languages:
+            full_path = os.path.join(prefix, 'html_%s' % (language), 'index.html')
+            if os.path.isfile(full_path):
+                dbg('Found %s manual' % (language))
+                return full_path
+            dbg('Couldn\'t find manual for %s language' % (language))
+
+    full_path = os.path.join(prefix, 'html', 'index.html')
+    if os.path.isfile(full_path):
+        dbg('Falling back to the default manual')
+        return full_path
+    else:
+        dbg('I can\'t find any suitable manual')
+        return None
+
 def path_lookup(command):
     '''Find a command in our path'''
     if os.path.isabs(command):
@@ -124,8 +139,7 @@ def shell_lookup():
         usershell = pwd.getpwuid(os.getuid())[6]
     except KeyError:
         usershell = None
-    shells = [os.getenv('SHELL'), usershell, 'bash',
-            'zsh', 'tcsh', 'ksh', 'csh', 'sh']
+    shells = [usershell, 'bash', 'zsh', 'tcsh', 'ksh', 'csh', 'sh']
 
     for shell in shells:
         if shell is None:
@@ -176,7 +190,13 @@ def get_config_dir():
 def dict_diff(reference, working):
     """Examine the values in the supplied working set and return a new dict
     that only contains those values which are different from those in the
-    reference dictionary"""
+    reference dictionary
+    
+    >>> a = {'foo': 'bar', 'baz': 'bjonk'}
+    >>> b = {'foo': 'far', 'baz': 'bjonk'}
+    >>> dict_diff(a, b)
+    {'foo': 'far'}
+    """
 
     result = {}
 
@@ -192,28 +212,34 @@ def get_edge(allocation, direction):
     directional navigation"""
     if direction == 'left':
         edge = allocation.x
+        p1, p2 = allocation.y, allocation.y + allocation.height
     elif direction == 'up':
         edge = allocation.y
+        p1, p2 = allocation.x, allocation.x + allocation.width
     elif direction == 'right':
         edge = allocation.x + allocation.width
+        p1, p2 = allocation.y, allocation.y + allocation.height
     elif direction == 'down':
         edge = allocation.y + allocation.height
+        p1, p2 = allocation.x, allocation.x + allocation.width
     else:
         raise ValueError('unknown direction %s' % direction)
     
-    return(edge)
+    return(edge, p1, p2)
 
-def get_nav_possible(edge, allocation, direction):
+def get_nav_possible(edge, allocation, direction, p1, p2):
     """Check if the supplied allocation is in the right direction of the
     supplied edge"""
+    x1, x2 = allocation.x, allocation.x + allocation.width
+    y1, y2 = allocation.y, allocation.y + allocation.height
     if direction == 'left':
-        return((allocation.x + allocation.width) <= edge)
+        return(x2 <= edge and y1 <= p2 and y2 >= p1)
     elif direction == 'right':
-        return(allocation.x >= edge)
+        return(x1 >= edge and y1 <= p2 and y2 >= p1)
     elif direction == 'up':
-        return((allocation.y + allocation.height) <= edge)
+        return(y2 <= edge and x1 <= p2 and x2 >= p1)
     elif direction == 'down':
-        return(allocation.y >= edge)
+        return(y1 >= edge and x1 <= p2 and x2 >= p1)
     else:
         raise ValueError('Unknown direction: %s' % direction)
 
@@ -223,11 +249,11 @@ def get_nav_offset(edge, allocation, direction):
     if direction == 'left':
         return(edge - (allocation.x + allocation.width))
     elif direction == 'right':
-        return(edge + allocation.x)
+        return(allocation.x - edge)
     elif direction == 'up':
-        return(edge - (allocation.y - allocation.height))
+        return(edge - (allocation.y + allocation.height))
     elif direction == 'down':
-        return(edge + allocation.y)
+        return(allocation.y - edge)
     else:
         raise ValueError('Unknown direction: %s' % direction)
 
@@ -276,3 +302,32 @@ def enumerate_descendants(parent):
         len(terminals), parent))
     return(containers, terminals)
 
+def make_uuid(str_uuid=None):
+    """Generate a UUID for an object"""
+    if str_uuid:
+        return uuid.UUID(str_uuid)
+    return uuid.uuid4()
+
+def inject_uuid(target):
+    """Inject a UUID into an existing object"""
+    uuid = make_uuid()
+    if not hasattr(target, "uuid") or target.uuid == None:
+        dbg("Injecting UUID %s into: %s" % (uuid, target))
+        target.uuid = uuid
+    else:
+        dbg("Object already has a UUID: %s" % target)
+
+def spawn_new_terminator(cwd, args):
+    """Start a new terminator instance with the given arguments"""
+    cmd = sys.argv[0]
+
+    if not os.path.isabs(cmd):
+        # Command is not an absolute path. Figure out where we are
+        cmd = os.path.join (cwd, sys.argv[0])
+        if not os.path.isfile(cmd):
+            # we weren't started as ./terminator in a path. Give up
+            err('Unable to locate Terminator')
+            return False
+      
+    dbg("Spawning: %s" % cmd)
+    subprocess.Popen([cmd]+args)
