@@ -5,12 +5,13 @@
 
 import copy
 import time
+import uuid
 import pygtk
 pygtk.require('2.0')
 import gobject
 import gtk
 
-from util import dbg, err
+from util import dbg, err, make_uuid
 import util
 from translation import _
 from version import APP_NAME
@@ -37,6 +38,8 @@ class Window(Container, gtk.Window):
     losefocus_time = 0
     position = None
     ignore_startup_show = None
+    set_pos_by_ratio = None
+    last_active_term = None
 
     zoom_data = None
 
@@ -60,7 +63,7 @@ class Window(Container, gtk.Window):
         self.register_signals(Window)
 
         self.set_property('allow-shrink', True)
-        self.apply_icon()
+        icon_to_apply=''
 
         self.register_callbacks()
         self.apply_config()
@@ -70,17 +73,24 @@ class Window(Container, gtk.Window):
 
         options = self.config.options_get()
         if options:
-            if options.forcedtitle is not None:
+            if options.forcedtitle:
                 self.title.force_title(options.forcedtitle)
 
-            if options.role is not None:
+            if options.role:
                 self.set_role(options.role)
+            
+            if options.classname is not None:
+                self.set_wmclass(options.classname, self.wmclass_class)
+            
+            if options.forcedicon is not None:
+                icon_to_apply = options.forcedicon
 
-            if options.geometry is not None:
+            if options.geometry:
                 if not self.parse_geometry(options.geometry):
                     err('Window::__init__: Unable to parse geometry: %s' % 
                             options.geometry)
 
+        self.apply_icon(icon_to_apply)
         self.pending_set_rough_geometry_hint = False
 
     def do_get_property(self, prop):
@@ -109,18 +119,19 @@ class Window(Container, gtk.Window):
 
         # Attempt to grab a global hotkey for hiding the window.
         # If we fail, we'll never hide the window, iconifying instead.
-        try:
-            self.hidebound = keybinder.bind(
-                self.config['keybindings']['hide_window'],
-                self.on_hide_window)
-        except (KeyError, NameError):
-            pass
+        if self.config['keybindings']['hide_window'] != None:
+            try:
+                self.hidebound = keybinder.bind(
+                    self.config['keybindings']['hide_window'],
+                    self.on_hide_window)
+            except (KeyError, NameError):
+                pass
 
-        if not self.hidebound:
-            err('Unable to bind hide_window key, another instance/window has it.')
-            self.hidefunc = self.iconify
-        else:
-            self.hidefunc = self.hide
+            if not self.hidebound:
+                err('Unable to bind hide_window key, another instance/window has it.')
+                self.hidefunc = self.iconify
+            else:
+                self.hidefunc = self.hide
 
     def apply_config(self):
         """Apply various configuration options"""
@@ -155,16 +166,29 @@ class Window(Container, gtk.Window):
         else:
             self.set_iconified(hidden)
 
-    def apply_icon(self):
+    def apply_icon(self, requested_icon):
         """Set the window icon"""
-        icon_theme = gtk.IconTheme()
+        icon_theme = gtk.icon_theme_get_default()
+        icon_name_list = [self.wmclass_name, APP_NAME]
 
-        try:
-            icon = icon_theme.load_icon(APP_NAME, 48, 0)
-        except (NameError, gobject.GError):
-            dbg('Unable to load 48px Terminator icon')
-            icon = self.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
+        if requested_icon:
+            try:
+                self.set_icon_from_file(requested_icon)
+                return
+            except (NameError, gobject.GError):
+                dbg('Unable to load %s icon as file' % (repr(requested_icon)))
 
+            icon_name_list.insert(0, requested_icon)
+
+        for icon_name in icon_name_list:
+            # Test if the icon is available first
+            if icon_theme.lookup_icon(icon_name, 48, 0):
+                self.set_icon_name(icon_name)
+                return # Success! We're done.
+            else:
+                dbg('Unable to load %s icon' % (icon_name))
+
+        icon = self.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
         self.set_icon(icon)
 
     def on_key_press(self, window, event):
@@ -209,6 +233,8 @@ class Window(Container, gtk.Window):
     def on_focus_in(self, window, event):
         """Focus has entered the window"""
         self.set_urgency_hint(False)
+        if not self.terminator.doing_layout:
+            self.terminator.last_active_window = self.uuid
         # FIXME: Cause the terminal titlebars to update here
 
     def is_child_notebook(self):
@@ -219,6 +245,7 @@ class Window(Container, gtk.Window):
     def tab_new(self, widget=None, debugtab=False, _param1=None, _param2=None):
         """Make a new tab"""
         cwd = None
+        profile = None
 
         if self.get_property('term_zoomed') == True:
             err("You can't create a tab while a terminal is maximised/zoomed")
@@ -226,11 +253,15 @@ class Window(Container, gtk.Window):
 
         if widget:
             cwd = widget.get_cwd()
+            profile = widget.get_profile()
+
         maker = Factory()
         if not self.is_child_notebook():
             dbg('Making a new Notebook')
             notebook = maker.make('Notebook', window=self)
-        self.get_child().newtab(debugtab, cwd=cwd)
+        self.show()
+        self.present()
+        return self.get_child().newtab(debugtab, cwd=cwd, profile=profile)
 
     def on_delete_event(self, window, event, data=None):
         """Handle a window close request"""
@@ -246,10 +277,8 @@ class Window(Container, gtk.Window):
     def confirm_close(self, window, type):
         """Display a confirmation dialog when the user is closing multiple
         terminals in one window"""
-        dialog = self.construct_confirm_close(window, type)
-        result = dialog.run()
-        dialog.destroy()
-        return(not (result == gtk.RESPONSE_ACCEPT))
+        
+        return(not (self.construct_confirm_close(window, type) == gtk.RESPONSE_ACCEPT))
 
     def on_destroy_event(self, widget, data=None):
         """Handle window destruction"""
@@ -273,6 +302,12 @@ class Window(Container, gtk.Window):
             if self.position:
                 self.move(self.position[0], self.position[1])
             self.show()
+            self.grab_focus()
+            try:
+                t = gtk.gdk.x11_get_server_time(self.window)
+            except AttributeError:
+                t = 0
+            self.window.focus(t)
         else:
             self.position = self.get_position()
             self.hidefunc()
@@ -346,12 +381,15 @@ class Window(Container, gtk.Window):
     
     def show(self, startup=False):
         """Undo the startup show request if started in hidden mode"""
-        gtk.Window.show(self)
-        #Present is necessary to grab focus when window is hidden from taskbar
+        #Present is necessary to grab focus when window is hidden from taskbar.
+        #It is important to call present() before show(), otherwise the window
+        #won't be brought to front if an another application has the focus.
+        #Last note: present() will implicitly call gtk.Window.show()
         self.present()
 
         #Window must be shown, then hidden for the hotkeys to be registered
         if (self.ignore_startup_show and startup == True):
+            self.position = self.get_position()
             self.hide()
 
 
@@ -367,8 +405,10 @@ class Window(Container, gtk.Window):
                        'unzoom': self.unzoom,
                        'tab-change': self.tab_change,
                        'group-all': self.group_all,
+                       'group-all-toggle': self.group_all_toggle,
                        'ungroup-all': self.ungroup_all,
                        'group-tab': self.group_tab,
+                       'group-tab-toggle': self.group_tab_toggle,
                        'ungroup-tab': self.ungroup_tab,
                        'move-tab': self.move_tab,
                        'tab-new': [self.tab_new, widget],
@@ -420,11 +460,18 @@ class Window(Container, gtk.Window):
             container = maker.make('VPaned')
         else:
             container = maker.make('HPaned')
+        
+        self.set_pos_by_ratio = True
 
         if not sibling:
             sibling = maker.make('Terminal')
             sibling.set_cwd(cwd)
             sibling.spawn_child()
+            if widget.group and self.config['split_to_group']:
+                sibling.set_group(None, widget.group)
+        if self.config['always_split_with_profile']:
+            sibling.force_set_profile(None, widget.get_profile())
+
         self.add(container)
         container.show_all()
 
@@ -435,6 +482,12 @@ class Window(Container, gtk.Window):
         for term in order:
             container.add(term)
         container.show_all()
+        sibling.grab_focus()
+        
+        while gtk.events_pending():
+            gtk.main_iteration_do(False)
+        self.set_pos_by_ratio = False
+
 
     def zoom(self, widget, font_scale=True):
         """Zoom a terminal widget"""
@@ -479,6 +532,29 @@ class Window(Container, gtk.Window):
         self.zoom_data = None
         self.set_property('term_zoomed', False)
 
+    def rotate(self, widget, clockwise):
+        """Rotate children in this window"""
+        self.set_pos_by_ratio = True
+        maker = Factory()
+        # collect all paned children in breadth-first order
+        paned = []
+        for child in self.get_children():
+            if maker.isinstance(child, 'Paned'):
+                paned.append(child)
+        for p in paned:
+            for child in p.get_children():
+                if child not in paned and maker.isinstance(child, 'Paned'):
+                    paned.append(child)
+        # then propagate the rotation
+        for p in paned:
+            p.rotate(widget, clockwise)
+        self.show_all()
+        widget.grab_focus()
+        
+        while gtk.events_pending():
+            gtk.main_iteration_do(False)
+        self.set_pos_by_ratio = False
+
     def get_visible_terminals(self):
         """Walk down the widget tree to find all of the visible terminals.
         Mostly using Container::get_visible_terminals()"""
@@ -528,6 +604,8 @@ class Window(Container, gtk.Window):
     def set_rough_geometry_hints(self):
         """Walk all the terminals along the top and left edges to fake up how
         many columns/rows we sort of have"""
+        if self.ismaximised == True:
+            return
         if not hasattr(self, 'cached_maker'):
             self.cached_maker = Factory()
         maker = self.cached_maker
@@ -605,19 +683,29 @@ class Window(Container, gtk.Window):
         # change
         child.set_current_page(child.get_current_page())
 
-    # FIXME: All of these (un)group_(all|tab) methods need refactoring work
+    def set_groups(self, new_group,  term_list):
+        """Set terminals in term_list to new_group"""
+        for terminal in term_list:
+            terminal.set_group(None, new_group)
+        self.terminator.focus_changed(self.terminator.last_focused_term)
+
     def group_all(self, widget):
         """Group all terminals"""
         # FIXME: Why isn't this being done by Terminator() ?
         group = _('All')
         self.terminator.create_group(group)
-        for terminal in self.terminator.terminals:
-            terminal.set_group(None, group)
+        self.set_groups(group, self.terminator.terminals)
+
+    def group_all_toggle(self, widget):
+        """Toggle grouping to all"""
+        if widget.group == 'All':
+            self.ungroup_all(widget)
+        else:
+            self.group_all(widget)
 
     def ungroup_all(self, widget):
         """Ungroup all terminals"""
-        for terminal in self.terminator.terminals:
-            terminal.set_group(None, None)
+        self.set_groups(None, self.terminator.terminals)
 
     def group_tab(self, widget):
         """Group all terminals in the current tab"""
@@ -634,8 +722,14 @@ class Window(Container, gtk.Window):
             if group not in self.terminator.groups:
                 break
             pagenum += 1
-        for terminal in self.get_visible_terminals():
-            terminal.set_group(None, group)
+        self.set_groups(group, self.get_visible_terminals())
+
+    def group_tab_toggle(self, widget):
+        """Blah"""
+        if widget.group and widget.group[:4] == 'Tab ':
+            self.ungroup_tab(widget)
+        else:
+            self.group_tab(widget)
 
     def ungroup_tab(self, widget):
         """Ungroup all terminals in the current tab"""
@@ -646,8 +740,7 @@ class Window(Container, gtk.Window):
             dbg('note in a notebook, refusing to ungroup tab')
             return
         
-        for terminal in self.get_visible_terminals():
-            terminal.set_group(None, None)
+        self.set_groups(None, self.get_visible_terminals())
 
     def move_tab(self, widget, direction):
         """Handle a keyboard shortcut for moving tab positions"""
@@ -710,12 +803,13 @@ class Window(Container, gtk.Window):
             possibles = []
 
             # Get the co-ordinate of the appropriate edge for this direction
-            edge = util.get_edge(allocation, direction)
+            edge, p1, p2 = util.get_edge(allocation, direction)
             # Find all visible terminals which are, in their entirity, in the
-            # direction we want to move
+            # direction we want to move, and are at least partially spanning
+            # p1 to p2
             for term in layout:
                 rect = layout[term]
-                if util.get_nav_possible(edge, rect, direction):
+                if util.get_nav_possible(edge, rect, direction, p1, p2):
                     possibles.append(term)
 
             if len(possibles) == 0:
@@ -783,6 +877,12 @@ class Window(Container, gtk.Window):
 
         self.get_children()[0].create_layout(child)
 
+        if layout.has_key('last_active_term') and layout['last_active_term'] not in ['', None]:
+            self.last_active_term = make_uuid(layout['last_active_term'])
+
+        if layout.has_key('last_active_window') and layout['last_active_window'] == 'True':
+            self.terminator.last_active_window = self.uuid
+
 class WindowTitle(object):
     """Class to handle the setting of the window title"""
 
